----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 03.04.2014 09:42:28
-- Design Name: 
-- Module Name: SkewComp - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SkewComp is
    Port ( TXUSRCLK2 : in STD_LOGIC;
           TXDATA : out STD_LOGIC_VECTOR (63 downto 0);
           ENABLE : in STD_LOGIC;
           SKEWPATSELECT : in STD_LOGIC_VECTOR (3 downto 0);
           ZEROSTOADD : in STD_LOGIC_VECTOR (15 downto 0);
           OVERSAMPLINGFACTOR : in STD_LOGIC_VECTOR(4 downto 0));
end SkewComp;

architecture Behavioral of SkewComp is
begin
	SkewCompenstation_process : process (TXUSRCLK2) is
	subtype index_int_t is integer range 0 to 1665;
	subtype small_int_t is integer range 0 to 26;
		
	variable count : integer := 0;
	variable assigned : std_logic :='0';
	variable done : std_logic := '0';
	variable SKEWPAT : std_logic_vector(63 downto 0);
	variable TXDATAOVERSAMPLED : std_logic_vector(1664 downto 0) := (others => '0');
	variable ofactor : small_int_t:=0;
	variable oindex : index_int_t:=0;
	variable upindex : integer;
	variable lowindex : integer;
	
	begin
		if rising_edge(TXUSRCLK2) then
			if ENABLE = '1' then
				if done /='1' then
					if assigned = '0' then
						done := '0';
						ofactor := to_integer(unsigned(OVERSAMPLINGFACTOR));
						count := to_integer(unsigned(ZEROSTOADD));
						assigned := '1';
					end if;
					
					case SKEWPATSELECT is
						when "0000" => SKEWPAT :="0000000000000111110101010101010101010101010111110000000000010101";
						when "0001" => SKEWPAT :="0101010101010101010101010101010101010101010101010101010101011111";
						when "0010" => SKEWPAT :="1111111111111101010101010101000000000000000000000000000000010101";
						when "0011" => SKEWPAT :="0000111100000000000000000000000000000000000000000000000000000001";
						when "0100" => SKEWPAT :="0000111100000000000000000000000000000000000000000000000000000101";
						when "0101" => SKEWPAT :="1111000000001010000101000010100001010001010001010001010000000101";
						when others => SKEWPAT :="0000000000000000000000000000000000000000000000000000000000000000";
					end case;
					--Oversample the SKEWPATTERN and place it into the oversample array
					TXDATAOVERSAMPLED := (others => '0');
					for i in 63 downto 0 loop
						upindex := (i*ofactor + ofactor-1);
						lowindex := i*ofactor;
						if SKEWPAT(i) = '0' then
						TXDATAOVERSAMPLED(upindex downto lowindex) := (others => '0');
						else
						TXDATAOVERSAMPLED(upindex downto lowindex) := (others => '1');
						end if; 
					end loop;
									
					
					--Compensate for the zeros to add, so skew alignment can be rechecked after zeros are inputted
					if count >= 64 then 
						TXDATA <= "0000000000000000000000000000000000000000000000000000000000000000";
						count := count - 64;
					elsif count < 64 and count > 0 then
						TXDATA(count-1 downto 0) <= (others => '0');
						TXDATA(63 downto count) <= TXDATAOVERSAMPLED(63-count downto 0); 
						oindex := 63-count+1; --This is the following lowest index to play, if this index is equal to oversamplefactor*64 (1*64 = 64) then stop playing
						count := 0;
					elsif count=0 and oindex < ofactor * 64 then
						--Play the rest of TXDATABIG and add further zeros if necessary this is also default situation so if not passed previous if this should play full skewpat
						TXDATA <= TXDATAOVERSAMPLED(oindex+63 downto oindex);
						oindex:= oindex +64;
					else
						done := '1';
						TXDATA <= "0000000000000000000000000000000000000000000000000000000000000000";
						
					end if;
				else
					TXDATA <= "0000000000000000000000000000000000000000000000000000000000000000";
				end if;
			else
				assigned := '0';
				count := 0;
				oindex := 0;
				done := '0';
				TXDATA <= "0000000000000000000000000000000000000000000000000000000000000000";	
			end if;
		end if;
	end process SkewCompenstation_process;
	


end Behavioral;
